import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import biz.papercut.pcng.ext.paymentgateway.ExternalCreditSourceProvider;

/*
 * (c) Copyright 2007 PaperCut Software Int. Pty. Ltd.
 */

/**
 * This example external credit provides the function to allows students to transfer small amounts of money 
 * into their print account. The amounts are saved into a tab-delimited file - one file for each month.  At the end 
 * of the month/term/semester the file(s) are imported into the student management system and the values are added to 
 * their student fees. 
 */
public class StudentLoanExternalCreditSource implements ExternalCreditSourceProvider {
    
    private static double DEFAULT_MAX_ALLOWED_AMOUNT        = 10.00;
    private static String MAX_ALLOWED_AMOUNT_CONFIG_KEY     = "student-load-source.max-allowed-amount";
    private static String LOG_DIRECTORY                     = "C:\\";
    
    private Boolean _logFileLock = new Boolean(true);
    private Properties _configProperties = null;

    public String getSourceName() {
        return "Student Loan Service";
    }

    public void setConfig(Properties properties) {
        _configProperties = properties;
    }

    public RequestResponse withdrawCredit(String username, String fullName, String cardNumber, double amount,
            String transactionComment) {

        if (amount > getMaxAllowedAmount()) {
            return new RequestResponse(
                    false, 
                    "The amount requested exceeds the maximum allowed limit."
                    );
        }
        
        synchronized (_logFileLock) {
            
            /*
             *  Open log file (Not very efficent opening/closing the log file every time but trasaction load is likly 
             *  to be very light.)
             */
            BufferedWriter logWriter = getLogWritter();
            if (logWriter == null) {
                // Problem opening our log file.
                return new RequestResponse(
                        false, 
                        "An error has occurred [1001]. Please contact the system administrator."
                        );
            }

            // Write tab-delim file in format DATE, USERNAME, FULLNAME, AMOUNT
            try {
                SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
                String dateTime = df.format(new Date());
                logWriter.write(dateTime);
                logWriter.write('\t');
                
                logWriter.write(username);
                logWriter.write('\t');
                
                logWriter.write(fullName);
                logWriter.write('\t');
                
                NumberFormat nf = NumberFormat.getInstance();
                nf.setMaximumFractionDigits(2);
                nf.setMinimumFractionDigits(2);
                logWriter.write(nf.format(amount));
                
                logWriter.newLine();
                logWriter.close();
                
            } catch (Throwable t) {
                return new RequestResponse(
                        false, 
                        "An error has occurred [1002]. Please contact the system administrator."
                        );
            }
        }
       
        // Return Success
        return new RequestResponse(true, "");
    }

    public Double availableCredit(String username, String fullName, String cardNumber) {
        // Not implemented for this solution.
        return null;
    }
    
    /**
     * @return Returns the maxium amount we'll allow users to transfer/add.
     */
    private double getMaxAllowedAmount() {
        double maxAmount = DEFAULT_MAX_ALLOWED_AMOUNT;
        if (_configProperties != null) {
            String s = _configProperties.getProperty(MAX_ALLOWED_AMOUNT_CONFIG_KEY);
            if (s != null) {
                try {
                    maxAmount = Double.parseDouble(s);
                } catch (Throwable t){
                    // no action - just use default.
                }
            }
        }
        return maxAmount;
    }
    

    /**
     * @return Returns a <code>BufferedWriter</code> associated with our trasaction log.
     */
    private BufferedWriter getLogWritter() {
        BufferedWriter logWriter = null;
        File logHome = new File(LOG_DIRECTORY);
        logHome.mkdirs();
        // Open Event Log
        try {
            SimpleDateFormat df = new SimpleDateFormat("yyyy-MM");
            String logName = "print-loan-transactions-" + df.format(new Date());
            
            File logFile = new File(logHome, logName);
            logWriter = new BufferedWriter(new FileWriter(logFile, true));
        } catch (Exception e) {
            // No action.  Just return null.
        }
        return logWriter;
    }

}
